/*
 * Copyright (c) 1992-94 Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the name of Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
 * POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
/*
 * Copyright (C) 1992   Silicon Graphics, Inc.
 *
 _______________________________________________________________________
 ______________  S I L I C O N   G R A P H I C S   I N C .  ____________
 |
 |   $Revision: 1.1 $
 |
 |   Classes:
 |	SoPixmapButton
 |
 |   Author(s)	: Alain Dumesny
 |
 |
 ______________  S I L I C O N   G R A P H I C S   I N C .  ____________
 _______________________________________________________________________
 */

#include <X11/Intrinsic.h>
#include <Xm/Xm.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>

#include "SoPixmapButton.h"

////////////////////////////////////////////////////////////////////////
//
//  Constructor
//
SoPixmapButton::SoPixmapButton(Widget p, SbBool canSelect)
//
////////////////////////////////////////////////////////////////////////
{
    parent = p;
    selectFlag = FALSE;
    selectable = canSelect;
    normalPixmap = armPixmap = selectPixmap = 0;
    
    // Create the push button
    Arg args[8];
    int n = 0;
    XtSetArg(args[n], XmNmarginHeight, 0); n++;
    XtSetArg(args[n], XmNmarginWidth, 0); n++;
    XtSetArg(args[n], XmNshadowThickness, 2); n++;
    XtSetArg(args[n], XmNhighlightThickness, 0); n++;
    widget = XmCreatePushButtonGadget(parent, NULL, args, n);
}

////////////////////////////////////////////////////////////////////////
//
//  Destructor
//
SoPixmapButton::~SoPixmapButton()
//
////////////////////////////////////////////////////////////////////////
{
    //??? destroy the widget?
}

////////////////////////////////////////////////////////////////////////
//
//  Highlight the pixmap button.
//
//  Usage: public
//
void
SoPixmapButton::select(SbBool flag)
//
////////////////////////////////////////////////////////////////////////
{
    if (selectFlag == flag || ! selectable)
	return;
    
    selectFlag = flag;
    
    XtVaSetValues(widget, XmNlabelPixmap, 
	selectFlag ? selectPixmap : normalPixmap, NULL);
}

////////////////////////////////////////////////////////////////////////
//
//  This routine builds the pixmaps (label pixmap and arm pixmap).
//
//  Usage: public
//
void
SoPixmapButton::setIcon(char *icon, int width, int height)
//
////////////////////////////////////////////////////////////////////////
{
    Display	*display = XtDisplay(parent);
    Drawable	d = DefaultRootWindow(display);
    int		depth = XDefaultDepthOfScreen(XtScreen(parent));
    Pixel	fg, bg, hl, abg;
    Arg		args[8];
    int		n;
    
    // get the color of the push buttons
    // ??? the foregrounf and background color have to be
    // ??? taken from the parent widget because we are using
    // ??? Gadget push buttons (not Widget push buttons)
    n = 0;
    XtSetArg(args[n], XmNforeground, &fg); n++;
    XtSetArg(args[n], XmNbackground, &bg); n++;
    XtSetArg(args[n], XmNtopShadowColor, &hl); n++; //??? highlight color
    XtGetValues(parent, args, n);
    
    n = 0;
    XtSetArg(args[n], XmNarmColor, &abg); n++;
    XtGetValues(widget, args, n);
    
    // free the old pixmaps
    if (normalPixmap)
	XFreePixmap(display, normalPixmap);
    if (armPixmap)
	XFreePixmap(display, armPixmap);
    if (selectPixmap)
	XFreePixmap(display, selectPixmap);
    normalPixmap = armPixmap = selectPixmap = 0;
    
    // create the pixmaps from the bitmap data (depth 1).
    normalPixmap = XCreatePixmapFromBitmapData(display, d, 
	icon, width, height, fg, bg, depth);
    armPixmap = XCreatePixmapFromBitmapData(display, d, 
	icon, width, height, fg, abg, depth);
    if (selectable)
	selectPixmap = XCreatePixmapFromBitmapData(display, d, 
	    icon, width, height, fg, hl, depth);
    
    // now assign the pixmaps
    n = 0;
    XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
    XtSetArg(args[n], XmNlabelPixmap, 
	selectFlag ? selectPixmap : normalPixmap); n++;
    XtSetArg(args[n], XmNarmPixmap, armPixmap); n++;
    XtSetValues(widget, args, n);
}

